@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cmdids.h,v 1.28 2009/10/22 00:00:17 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/cmdids.h,v 1.34 2011/05/16 16:21:56 william Exp $
  */
 
 #ifndef _CMDIDS_H_
@@ -440,11 +440,15 @@
 #define CMDID_EXTENDSEGMENT           498
 #define CMDID_TOGGLETIGHTERSPLINES    499
 
-#define CMDID_INSERTSYMBOL             500
-#define CMDID_TOGGLERIGHTMARGIN        501
-#define CMDID_SPECIFYRIGHTMARGIN       502
-#define CMDID_CHANGEALLSELFONTOVERLINE 503
+#define CMDID_INSERTSYMBOL                  500
+#define CMDID_TOGGLERIGHTMARGIN             501
+#define CMDID_SPECIFYRIGHTMARGIN            502
+#define CMDID_CHANGEALLSELFONTOVERLINE      503
+#define CMDID_TOGGLEFLOODREPLACECOLORTHRESH 504
+#define CMDID_SETFLOODREPLACECOLORTHRESH    505
+#define CMDID_REMOVETRANSPIXEL              506
+#define CMDID_REPLACECOLORWITHTRANS         507
 
-#define MAXCMDIDS 504
+#define MAXCMDIDS 508
 
 #endif /*_CMDIDS_H_*/
