@@ -15,7 +15,7 @@
  * NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION
  * WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
- * @(#)$Header: /mm2/home/cvs/bc-src/tgif/setup.c,v 1.58 2009/09/20 23:42:49 cvsps Exp $
+ * @(#)$Header: /mm2/home/cvs/bc-src/tgif/setup.c,v 1.82 2011/05/19 21:59:45 william Exp $
  */
 
 #define _INCLUDE_FROM_SETUP_C_
@@ -78,6 +78,7 @@
 #include "tdgtlist.e"
 #include "text.e"
 #include "tgcwheel.e"
+#include "tginssym.e"
 #include "tidget.e"
 #include "util.e"
 #include "version.e"
@@ -85,6 +86,7 @@
 #include "xbitmap.e"
 #include "xpixmap.e"
 #include "xprtfltr.e"
+#include "z_intrf.e"
 
 #include "tgificon.xbm"
 
@@ -119,6 +121,7 @@
 #define CHAT_WINDOW_H 40
 
 int	debugNoPointerGrab=FALSE;
+int	debugCopyPaste=FALSE;
 
 unsigned int	mainWinW = 0;
 unsigned int	mainWinH = 0;
@@ -511,6 +514,7 @@ void DoReconfigure()
    if (pageWindowW > 0) {
       XMoveResizeWindow(mainDisplay, pageWindow, 0, cur_y, pageWindowW,
             scrollBarW+(brdrW<<1));
+      MakeCurPageTabVisible();
    } else {
       XMoveWindow(mainDisplay, pageWindow, 0, cur_y);
    }
@@ -684,8 +688,7 @@ void HandleSelectionClear(input)
    selection = input->xselectionclear.selection;
    time = input->xselectionclear.time;
 
-#ifdef NOT_DEFINED
-   {
+   if (debugCopyPaste) {
       static int id=0;
       char *atom_name=NULL;
 
@@ -695,7 +698,6 @@ void HandleSelectionClear(input)
       printf("%1d: Selection name is '%s'.\n", id, atom_name);
       XFree(atom_name);
    }
-#endif /* NOT_DEFINED */
    if (startSelectionOwnershipTimeValid) {
       endSelectionOwnershipTime = time;
       endSelectionOwnershipTimeValid = TRUE;
@@ -703,6 +705,35 @@ void HandleSelectionClear(input)
    ClearSelection();
 }
 
+static
+void DebugSelectRequest(property, target, level, msg)
+   Atom property, target;
+   int level;
+   char *msg;
+{
+   if (debugCopyPaste > level) {
+      /*
+       * After tgif does a Copy, tgif will get SelectionRequest at
+       *         a regular interval
+       */
+      static int id=0;
+      char *atom_name=NULL;
+
+      if (msg == NULL) {
+         printf("\n%1d: SelectionRequest.\n", ++id);
+      } else {
+         printf("\n%1d: SelectionRequest (%s).\n", ++id, msg);
+      }
+      atom_name = XGetAtomName(mainDisplay, property);
+      printf("%1d: Property name is '%s'.\n", id, atom_name);
+      XFree(atom_name);
+
+      atom_name = XGetAtomName(mainDisplay, target);
+      printf("%1d: Target name is '%s'.\n", id, atom_name);
+      XFree(atom_name);
+   }
+}
+
 void HandleSelectionRequest(input)
    XEvent *input;
 {
@@ -710,33 +741,17 @@ void HandleSelectionRequest(input)
    Atom property=(Atom)0, target=(Atom)0, selection=(Atom)0;
    Time time=(Time)0;
    XEvent sent_ev;
+   unsigned char empty_buf[1];
 
+   *empty_buf = '\0';
    requestor = input->xselectionrequest.requestor;
    selection = input->xselectionrequest.selection;
    target = input->xselectionrequest.target;
    property = input->xselectionrequest.property;
    time = input->xselectionrequest.time;
 
-#ifdef NOT_DEFINED
-   {
-      /*
-       * After tgif does a Copy, tgif will get SelectionRequest at
-       *         a regular interval
-       */
-      static int id=0;
-      char *atom_name=NULL;
-
-      printf("\n%1d: SelectionRequest.\n", ++id);
-
-      atom_name = XGetAtomName(mainDisplay, property);
-      printf("%1d: Property name is '%s'.\n", id, atom_name);
-      XFree(atom_name);
-
-      atom_name = XGetAtomName(mainDisplay, target);
-      printf("%1d: Target name is '%s'.\n", id, atom_name);
-      XFree(atom_name);
-   }
-#endif /* NOT_DEFINED */
+   /* do not translate -- program constants */
+   DebugSelectRequest(property, target, 1, "function entry point");
    memset(&sent_ev, 0, sizeof(XEvent));
 
    sent_ev.xselection.type = SelectionNotify;
@@ -753,32 +768,77 @@ void HandleSelectionRequest(input)
        *         object copied into (i.e., what are the possobiel targets)?
        */
       Atom targets[5];
+      int num_targets=0;
 
-      targets[0] = XA_STRING;
-      targets[1] = compoundTextAtom;
-      targets[2] = tgifProtocolAtom;
-      targets[3] = textAtom;
-      targets[4] = utf8StringAtom;
+      DebugSelectRequest(property, target, 0, "TARGETS");
+      if (gSetCutBufferInfo.tgif_valid) {
+         targets[num_targets++] = tgifProtocolAtom;
+      }
+      if (gSetCutBufferInfo.utf8_valid) {
+         targets[num_targets++] = utf8StringAtom;
+      }
+      if (gSetCutBufferInfo.text_valid) {
+         targets[num_targets++] = compoundTextAtom;
+         targets[num_targets++] = textAtom;
+         targets[num_targets++] = XA_STRING;
+      }
       XChangeProperty(mainDisplay, requestor, property, target, 32,
-            PropModeReplace, (unsigned char *)targets, 5);
+            PropModeReplace, (unsigned char *)targets, num_targets);
       sent_ev.xselection.property = property;
    } else if (target == XA_STRING || target == textAtom) {
-      XChangeProperty(mainDisplay, requestor, property, target, 8,
-            PropModeReplace, (unsigned char *)(dsCutBuffer.s), dsCutBuffer.sz);
+      DebugSelectRequest(property, target, 0, "XA_STRING or TEXT");
+      if (gSetCutBufferInfo.text_valid) {
+         XChangeProperty(mainDisplay, requestor, property, target, 8,
+               PropModeReplace,
+               (unsigned char *)(gSetCutBufferInfo.text_dyn_str.s),
+               gSetCutBufferInfo.text_dyn_str.sz);
+      } else {
+         XChangeProperty(mainDisplay, requestor, property, target, 8,
+               PropModeReplace, empty_buf, 0);
+      }
       sent_ev.xselection.property = property;
    } else if (target == utf8StringAtom) {
-      XChangeProperty(mainDisplay, requestor, property, target, 8,
-            PropModeReplace, (unsigned char *)(dsCutBuffer.s), dsCutBuffer.sz);
+      DebugSelectRequest(property, target, 0, "UTF8_STRING");
+      if (gSetCutBufferInfo.utf8_valid) {
+         XChangeProperty(mainDisplay, requestor, property, target, 8,
+               PropModeReplace,
+               (unsigned char *)(gSetCutBufferInfo.utf8_dyn_str.s),
+               gSetCutBufferInfo.utf8_dyn_str.sz);
+      } else {
+         XChangeProperty(mainDisplay, requestor, property, target, 8,
+               PropModeReplace, empty_buf, 0);
+      }
       sent_ev.xselection.property = property;
    } else if (target == tgifProtocolAtom) {
-      XChangeProperty(mainDisplay, requestor, property, target, 8,
-            PropModeReplace, (unsigned char *)(dsCutBuffer.s), dsCutBuffer.sz);
+      DebugSelectRequest(property, target, 0, "_TGIF_PROTOCOL_ATOM");
+      if (gSetCutBufferInfo.tgif_valid) {
+         XChangeProperty(mainDisplay, requestor, property, target, 8,
+               PropModeReplace,
+               (unsigned char *)(gSetCutBufferInfo.tgif_dyn_str.s),
+               gSetCutBufferInfo.tgif_dyn_str.sz);
+      } else {
+         XChangeProperty(mainDisplay, requestor, property, target, 8,
+               PropModeReplace, empty_buf, 0);
+      }
       sent_ev.xselection.property = property;
    } else if (target == compoundTextAtom) {
       XTextProperty text_property;
 
-      XmbTextListToTextProperty(mainDisplay, &(dsCutBuffer.s), 1,
-            XCompoundTextStyle, &text_property);
+      if (gSetCutBufferInfo.utf8_valid) {
+         DebugSelectRequest(property, target, 0, "COMPOUND_TEXT for UTF8");
+         Xutf8TextListToTextProperty(mainDisplay,
+               &(gSetCutBufferInfo.utf8_dyn_str.s), 1,
+               XCompoundTextStyle, &text_property);
+      } else if (gSetCutBufferInfo.text_valid) {
+         DebugSelectRequest(property, target, 0, "COMPOUND_TEXT not UTF8");
+         XmbTextListToTextProperty(mainDisplay,
+               &(gSetCutBufferInfo.text_dyn_str.s), 1,
+               XCompoundTextStyle, &text_property);
+      } else {
+         DebugSelectRequest(property, target, 0, "COMPOUND_TEXT (none)");
+         XmbTextListToTextProperty(mainDisplay, (char**)(&empty_buf), 1,
+               XCompoundTextStyle, &text_property);
+      }
       XChangeProperty(mainDisplay, requestor, property, target,
             text_property.format, PropModeReplace, text_property.value,
             text_property.nitems);
@@ -889,7 +949,7 @@ int mainWinEventHandler(input)
                   TranslateKeys(buf, &key_sym);
                }
                /*
-                * This is probabl not needed.
+                * This is probably not needed.
                 *
                 * ClearCopyUTF8Info(); 
                 * if (textHighlight) {
@@ -1009,6 +1069,7 @@ int GetWMIconPixmap(pPixmap, pPixmapMask)
       for (i=0; i < ncolors; i++) free(color_str[i]);
       free(color_str);
    }
+   if (pixels != NULL) free(pixels);
    *pPixmap = pixmap;
    *pPixmapMask = bitmap;
    return TRUE;
@@ -1121,9 +1182,9 @@ void Setup()
 {
    int bitmask=0, x_neg_in_def=FALSE, y_neg_in_def=FALSE;
    int x_in_cmdline=FALSE, y_in_cmdline=FALSE, cur_y=0, i, x_offset=0;
-   int bg_pixel=(-1);
+   int bg_pixel=(-1), def_x_neg=0, def_y_neg=0, paper_size_set=FALSE;
    char *c_ptr=NULL;
-   int def_x_neg=0, def_y_neg=0, paper_size_set;
+   double dval=(double)0;
    XWMHints wmhints;
    XSizeHints sizehints;
    XClassHint classhints;
@@ -1485,6 +1546,9 @@ void Setup()
    if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"InitialPaperSize")) != NULL) {
       paper_size_set = SetPaperSize(c_ptr);
    }
+   if (cmdLineLetter) {
+      paper_size_set = SetPaperSize("letter");
+   }
    pageStyle = PORTRAIT;
    if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"PageStyleLandscape")) !=
          NULL && UtilStrICmp("true", c_ptr) == 0) {
@@ -1715,9 +1779,19 @@ void Setup()
          mainVisual->class==StaticGray) ? FALSE : TRUE;
    fullTrueColorMode = FALSE;
    if (colorDisplay && mainVisual->class == TrueColor) {
-      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"EnableTrueColorImages")) !=
-            NULL && UtilStrICmp(c_ptr, "true") == 0) {
+      if (HasZlibSupport()) {
+         /* starting with tgif-4.2.3, this is the default */
          fullTrueColorMode = TRUE;
+#ifdef _TGIF_DBG
+      } else {
+         fprintf(stderr, TgLoadString(STID_CANNOT_SET_XDEF_TRUE_NO_ZLIB),
+               TOOL_NAME, "EnableTrueColorImages", TOOL_NAME);
+         fprintf(stderr, "\n");
+#endif /* _TGIF_DBG */
+      }
+      if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"EnableTrueColorImages")) !=
+            NULL && UtilStrICmp(c_ptr, "false") == 0) {
+         fullTrueColorMode = FALSE;
       }
    }
    noAntiAlias = TRUE;
@@ -1767,6 +1841,42 @@ void Setup()
          "Btn2PopupMainMenu")) != NULL && UtilStrICmp(c_ptr, "false") == 0) {
       btn2PopupMainMenu = FALSE;
    }
+   defRightMarginEnabled = INVALID;
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"EnableRightMargin")) != NULL) {
+      if (UtilStrICmp("true", c_ptr) == 0) {
+         defRightMarginEnabled = TRUE;
+      } else {
+         defRightMarginEnabled = FALSE;
+      }
+   }
+   if (gridSystem == ENGLISH_GRID) {
+      defRightMargin = ONE_INCH;
+   } else {
+      defRightMargin = (ONE_CM<<1)+(ONE_CM>>1);
+   }
+   dval = ((double)defRightMargin) * ((double)100) * ((double)TIK_PER_PIXEL) /
+         ((double)printMag);
+   defRightMargin = round(dval);
+   if ((c_ptr=XGetDefault(mainDisplay,TOOL_NAME,"RightMargin")) != NULL) {
+      char spec_buf[40];
+
+      UtilStrCpyN(spec_buf, sizeof(spec_buf), c_ptr);
+      UtilTrimBlanks(spec_buf);
+      if (GetDimensionInDouble(spec_buf, TRUE, &dval)) {
+         dval = dval * ((double)100) * ((double)TIK_PER_PIXEL) /
+               ((double)printMag);
+         defRightMargin = round(dval);
+      } else {
+         fprintf(stderr, TgLoadString(STID_INVALID_XDEF_USE_ALT_STR),
+               TOOL_NAME, "RightMargin", c_ptr,
+               (gridSystem==ENGLISH_GRID) ? "1 in" : "2.5 cm");
+         fprintf(stderr, "\n");
+      }
+   }
+   rightMargin = defRightMargin;
+   rightMarginEnabled = defRightMarginEnabled;
+   rightMarginActive = (rightMarginEnabled == TRUE);
+
    InitFonts();
 
    initialMenubarWindowH = defaultFontHeight;
@@ -2065,6 +2175,7 @@ void Setup()
    }
    InitTidget();
    InitColorWheel();
+   InitInsertSymbol();
 
    if (!CreateChatSubWindows()) {
       FailToCreateWindowMessage("Setup()", NULL, TRUE);
